@@ -156,24 +156,6 @@ public:
   /** \overload */
   const Array<signed char>& formalCharges() const;
 
-  /**
-   * Get the total charge on the molecule.
-   * The method will first check to see if a total charge has been set. If not,
-   * it will calculate the total charge from the formal charges (if set).
-   * If neither has been set, it will assume the total charge is zero.
-   * @return The total charge of the molecule.
-   */
-  signed char totalCharge() const;
-
-  /**
-   * Get the total spin multiplicity of the molecule.
-   * The method will first check to see if a total spin has been set. If not,
-   * it will either suggest a singlet if an even number of electrons are
-   * present, or a doublet if an odd number of electrons are present.
-   * @return The total spin multiplicity of the molecule.
-   */
-  char totalSpinMultiplicity() const;
-
   /**
    * Get the formal charge for the requested atom.
    * @param atomId The index of the atom.
@@ -197,6 +179,53 @@ public:
    */
   bool setFormalCharge(Index atomId, signed char charge);
 
+  /**
+   * Get the total charge on the molecule.
+   * The method will first check to see if a total charge has been set. If not,
+   * it will calculate the total charge from the formal charges (if set).
+   * If neither has been set, it will assume the total charge is zero.
+   * @return The total charge of the molecule.
+   */
+  signed char totalCharge() const;
+
+  /**
+   * Get the total spin multiplicity of the molecule.
+   * The method will first check to see if a total spin has been set. If not,
+   * it will either suggest a singlet if an even number of electrons are
+   * present, or a doublet if an odd number of electrons are present.
+   * @return The total spin multiplicity of the molecule.
+   */
+  char totalSpinMultiplicity() const;
+
+  /** @return an array of the isotopes for the atoms in the molecule. */
+  Array<unsigned short>& isotopes();
+
+  /** \overload */
+  const Array<unsigned short>& isotopes() const;
+
+  /**
+   * Get the isotope for the requested atom.
+   * @param atomId The index of the atom.
+   * @return The isotope of the atom indexed at @a atomId, or
+   * 0 if @a atomId is invalid.
+   */
+  unsigned short isotope(Index atomId) const;
+
+  /**
+   * Replace the current array of isotopes.
+   * @param charges The new isotope array. Must be of length atomCount().
+   * @return True on success, false otherwise.
+   */
+  bool setIsotopes(const Core::Array<unsigned short>& isotopes);
+
+  /**
+   * Set the isotope of a single atom.
+   * @param atomId The index of the atom to modify.
+   * @param isotope The new isotope (or zero for most common)
+   * @return True on success, false otherwise.
+   */
+  bool setIsotope(Index atomId, unsigned short isotope);
+
   /** \returns a vector of colors for the atoms in the moleucle. */
   Array<Vector3ub>& colors();
 
@@ -440,8 +469,8 @@ public:
   const Core::Array<std::string> bondLabels() const { return m_bondLabels; }
 
   /**
-   * Set all the atom labels in the molecule.
-   * @param label The new label array. Must be of length atomCount().
+   * Set all the bond labels in the molecule.
+   * @param label The new label array. Must be of length bondCount().
    * @return True on success, false otherwise.
    */
   bool setBondLabels(const Core::Array<std::string>& label);
@@ -482,6 +511,26 @@ public:
 
   void clearCubes();
 
+  /**
+   * @brief Set the active cube index for volume rendering.
+   * @param index The index of the cube to make active.
+   * @return True if the index is valid and was set, false otherwise.
+   */
+  bool setActiveCubeIndex(Index index);
+
+  /**
+   * @brief Get the index of the active cube.
+   * @return The active cube index, or 0 if no cubes exist.
+   */
+  Index activeCubeIndex() const { return m_activeCubeIndex; }
+
+  /**
+   * @brief Get the active cube for volume rendering.
+   * @return A pointer to the active cube, or nullptr if no cubes exist.
+   */
+  Cube* activeCube();
+  const Cube* activeCube() const;
+
   /**
    * @brief Get the cubes vector set (if present) for the molecule.
    * @return The cube vector for the molecule
@@ -503,6 +552,13 @@ public:
    */
   double mass() const;
 
+  /**
+   * @return the monoisotopic mass of the molecule obtained by summing
+   * the isotopic masses of the atoms (including the most common isotope
+   * if not specified).
+   */
+  double monoisotopicMass() const;
+
   /**
    * @return The center of geometry of the molecule obtained by summing the
    * coordinates of the atoms.
@@ -647,6 +703,7 @@ public:
   Residue& addResidue(std::string& name, Index& number, char& id);
   void addResidue(Residue& residue);
   Residue& residue(Index index);
+  const Residue& residue(Index index) const;
 
   Array<Residue>& residues() { return m_residues; }
   const Array<Residue>& residues() const { return m_residues; }
@@ -654,6 +711,31 @@ public:
   /** @return The number of residues in the molecule. */
   Index residueCount() const;
 
+  /**
+   * @return Any custom label for the requested residue.
+   * @param residueIndex The index of the residue.
+   */
+  std::string residueLabel(Index residueIndex) const;
+  /**
+   * Set the custom label of a single residue.
+   * @param residueIndex The index of the residue to modify.
+   * @param label The new label of the residue.
+   * @return True on success, false otherwise.
+   */
+  bool setResidueLabel(Index residueIndex, const std::string& label);
+
+  const Core::Array<std::string> residueLabels() const
+  {
+    return m_residueLabels;
+  }
+
+  /**
+   * Set all the residue labels in the molecule.
+   * @param label The new label array. Must be of length residueCount().
+   * @return True on success, false otherwise.
+   */
+  bool setResidueLabels(const Core::Array<std::string>& label);
+
   /**  @return The number of atoms in the molecule. */
   Index atomCount() const;
 
@@ -814,6 +896,13 @@ public:
    */
   std::map<unsigned char, size_t> composition() const;
 
+  /**
+   * @return a map of element symbols (including isotopes like D, T, 13C) to
+   * their count. Handles unit cell fractional contributions for atoms at
+   * corners (1/8), edges (1/4), and faces (1/2).
+   */
+  std::map<std::string, size_t> formulaComposition() const;
+
   /**
    * @return the atom pairs for all bonds to the atom indexed at @a index.
    */
@@ -864,10 +953,12 @@ protected:
   Array<Vector3> m_positions3d;
   Array<std::string> m_atomLabels;
   Array<std::string> m_bondLabels;
+  Array<std::string> m_residueLabels;
   Array<Array<Vector3>> m_coordinates3d; //!< Store conformers/trajectories.
   Array<double> m_timesteps;
   Array<AtomHybridization> m_hybridizations;
   Array<signed char> m_formalCharges;
+  Array<unsigned short> m_isotopes; //!< Store isotopes of the atoms
   Array<Vector3> m_forceVectors;
   Array<Vector3ub> m_colors;
   // Vibration data if available.
@@ -881,6 +972,7 @@ protected:
 
   std::vector<Mesh*> m_meshes;
   std::vector<Cube*> m_cubes;
+  Index m_activeCubeIndex = 0;
 
   BasisSet* m_basisSet;
   UnitCell* m_unitCell;
@@ -970,6 +1062,31 @@ inline bool Molecule::setFormalCharge(Index atomId, signed char charge)
   return false;
 }
 
+inline unsigned short Molecule::isotope(Index atomId) const
+{
+  return atomId < m_isotopes.size() ? m_isotopes[atomId] : 0;
+}
+
+inline bool Molecule::setIsotopes(const Core::Array<unsigned short>& isotopes)
+{
+  if (isotopes.size() == atomCount()) {
+    m_isotopes = isotopes;
+    return true;
+  }
+  return false;
+}
+
+inline bool Molecule::setIsotope(Index atomId, unsigned short isotope)
+{
+  if (atomId < atomCount()) {
+    if (atomId >= m_isotopes.size())
+      m_isotopes.resize(atomCount(), 0);
+    m_isotopes[atomId] = isotope;
+    return true;
+  }
+  return false;
+}
+
 inline const Molecule::ElementMask Molecule::elements() const
 {
   return m_elements;
@@ -1177,7 +1294,7 @@ inline std::string Molecule::bondLabel(Index bondId) const
 
 inline bool Molecule::setBondLabels(const Core::Array<std::string>& labels)
 {
-  if (labels.size() == atomCount() || labels.size() == 0) {
+  if (labels.size() == bondCount() || labels.size() == 0) {
     m_bondLabels = labels;
     return true;
   }
@@ -1194,6 +1311,7 @@ inline bool Molecule::setBondLabel(Index bondId, const std::string& label)
   }
   return false;
 }
+
 inline const Graph& Molecule::graph() const
 {
   return m_graph;
